/***********************************************************************

  ColorBox2: utils.h
    (c) 2004 by Martin Decky

  Common datatypes, classes, macros.
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			10th August 2004		complete rewrite, C++
  0.4			10th September 2004		reverse iterators
  0.5			15th September 2004		this_types, storage_types, cleanup

***********************************************************************/


#ifndef __CB2__UTILS_H__
#define __CB2__UTILS_H__


#include <math.h>
#include <string>
#include <ostream>


/** Exception macros
 *
 * If the condition 'cond' is false,
 * the given exception is thrown
 * (with mandantory warning or value
 * identification).
 *
 */
#define CB2_ASSERT(cond, warning) \
	if (!(cond)) \
		throw ExceptionAssert(warning ": " #cond " in " __FILE__, __LINE__);


#define CB2_CONVERSION(cond, warning) \
	if (!(cond)) \
		throw ExceptionConversion(warning ": " #cond " in " __FILE__, __LINE__);
		

#define CB2_RANGECHECK(cond, value) \
	if (!(cond)) \
		throw ExceptionRange< typeof(value)>(#cond ": Value " #value "=", value, " is out of range in " __FILE__, __LINE__);
		
		
#define CB2_MEMORYCHECK(cond) \
	if (!(cond)) \
		throw ExceptionMemory(#cond ": Memory dereference fault in " __FILE__, __LINE__);


#define CB2_NOTFOUND(cond) \
	if (!(cond)) \
		throw ExceptionNotFound(#cond ": Item not found in " __FILE__, __LINE__);


/** ColorBox2 namespace
 *
 * All ColorBox2 types and classes
 * are within the CB2 namespace.
 *
 */
namespace CB2 {
	
	
	/** Base class of all ColorBox2 classes
	 *
	 * Common methods and properties.
	 *
	 */
	class Class {
		public:
			int tag; /**< Tag for user-specific data */
	};
	
	
	template< class Tsize> class ExceptionRange; /**< Forward declaration */
	class ExceptionConversion;                   /**< Forward declaration */
	class ExceptionMemory;                       /**< Forward declaration */
	class ExceptionAssert;                       /**< Forward declaration */
	class ExceptionNotFound;                     /**< Forward declaration */
	
	
	/** Generic iterator class
	 *
	 * This template implements a generic iterator
	 * for data items from Tcontainer. The iterator
	 * refers to the items by Tref and returns
	 * the values as Tmemb.
	 *
	 * Tcontainer must implement these methods:
	 *
	 *   - Tref RefNext(const Tref &)
	 *       To the given reference generate
	 *       the reference to the next item.
	 *       Should not throw any exceptions
	 *       (the successor of the final or invalid
	 *       reference should be the final reference).
	 *
	 *   - Tmemb Deref(const Tref &)
	 *       Get the value of the referenced
	 *       item. Might throw any useful exception
	 *       on final or invalid reference.
	 *
	 */
	template< class Tcontainer, class Tref, class Tmemb> class Iterator : public Class {
		public:
		
			typedef Iterator< Tcontainer, Tref, Tmemb> this_type; /**< Template type alias */
			
			
			/** Constructor
			 *
			 * Create an iterator refering
			 * to the given item.
			 *
			 * @param cont      Container.
			 * @param reference Reference to an item.
			 *
			 */
			Iterator(const Tcontainer & cont, const Tref & reference)
				: container(cont) {
				ref = reference;
			}
			
			
			/** Advance iterator reference
			 *
			 * Use container's RefNext() to get
			 * the reference to the succeding item.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator++() {
				ref = container.RefNext(ref);
				return *this;
			}
			
			
			/** Compare iterators
			 *
			 * Compare iterators by comparing
			 * the references.
			 *
			 * @param it Iterator to compare with.
			 *
			 * @return Compare result.
			 *
			 */
			bool operator==(const this_type & it) const {
				return (ref == it.ref);
			}
			
			bool operator!=(const this_type & it) const {
				return (!operator==(it));
			}
			
			
			/** Dereference iterators
			 *
			 * Get the item referenced by the iterator
			 * using container's Deref() method.
			 *
			 * @return Dereferenced value.
			 *
			 */
			const Tmemb & operator*() const {
				return container.Deref(ref);
			}
			
			const Tmemb * operator->() const {
				return &container.Deref(ref);
			}
			
		private:
			
			const Tcontainer & container; /**< Container reference */
			Tref ref;                     /**< Current item reference */
			
	};
	
	
	/** Generic reverse iterator class
	 *
	 * This template implements a generic reverse iterator
	 * for data items from Tcontainer. The reverse iterator
	 * refers to the items by Tref and returns
	 * the values as Tmemb.
	 *
	 * Tcontainer must implement these methods:
	 *
	 *   - Tref RefPrev(const Tref &)
	 *       To the given reference generate
	 *       the reference to the previous item.
	 *       Should not throw any exceptions
	 *       (the successor of the final or invalid
	 *       reference should be the final reference).
	 *
	 *   - Tmemb RevDeref(const Tref &)
	 *       Get the value of the referenced
	 *       item. Might throw any useful exception
	 *       on final or invalid reference.
	 *
	 */
	template< class Tcontainer, class Tref, class Tmemb> class RevIterator : public Class {
		public:
			
			typedef RevIterator< Tcontainer, Tref, Tmemb> this_type; /**< Template type alias */
			
			/** Constructor
			 *
			 * Create a reverse iterator refering
			 * to the given item.
			 *
			 * @param cont      Container.
			 * @param reference Reference to an item.
			 *
			 */
			RevIterator(const Tcontainer & cont, const Tref & reference)
				: container(cont) {
				ref = reference;
			}
			
			
			/** Unadvance reverse iterator reference
			 *
			 * Use container's RefNext() to get
			 * the reference to the succeding item.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator--() {
				ref = container.RefPrev(ref);
				return *this;
			}
			
			
			/** Compare reverse iterators
			 *
			 * Compare reverse iterators by comparing
			 * the references.
			 *
			 * @param it Iterator to compare with.
			 *
			 * @return Compare result.
			 *
			 */
			bool operator==(const this_type & it) const {
				return (ref == it.ref);
			}
			
			bool operator!=(const this_type & it) const {
				return (!operator==(it));
			}
			
			
			/** Dereference iterators
			 *
			 * Get the item referenced by the iterator
			 * using container's Deref() method.
			 *
			 * @return Dereferenced value.
			 *
			 */
			const Tmemb & operator*() const {
				return container.RevDeref(ref);
			}
			
			const Tmemb * operator->() const {
				return &container.RevDeref(ref);
			}
			
		private:
			
			const Tcontainer & container; /**< Container reference */
			Tref ref;                     /**< Current item reference */
			
	};
	
	
	/** Multipurpose dynamic array
	 *
	 * Multipurpose array type for storing values of
	 * the same kind. Grows in fragments, never
	 * shrinks by inself (shrinks only explicitly
	 * on Discart(true) method).
	 *
	 * Note: This is a low-level array class, the elements
	 * are stored and copied bit-by-bit (no copy constructors
	 * or class assignments are called). Use List
	 * for storing class objects.
	 *
	 * NB: As a low-level storage implementation, many
	 * operations have strict limitations on self-reference
	 * (i.e. you shouldn't insert values referenced from
	 * the same array instance). Use List for more robust
	 * (but slower) implementation.
	 *
	 */
	template< class Tmemb> class Array : public Class {
		public:
		
			typedef Array< Tmemb> this_type;                               /**< Template type alias */
			typedef size_t size_type;                                      /**< Size type */
			typedef Tmemb * ref_type;                                      /**< Item reference type */
			typedef Iterator< this_type, ref_type, Tmemb> iterator;        /**< Iterator */
			typedef RevIterator< this_type, ref_type, Tmemb> rev_iterator; /**< Reverse iterator */
		

			/** Default constructor
			 *
			 * Create empty array.
			 *
			 * @param fragment Default fragment size
			 *                 (in items count).
			 *
			 */
			Array(const size_type fragment = 4096) {
				/* Empty data */
				data = NULL;
				allocated = 0;
				frag = fragment;
				allocate(0);
			}
			
			
			/** Copy constructor
			 *
			 * Copy array items from another array
			 * (copy is bit-by-bit).
			 *
			 * @param prev Array to copy.
			 *
			 */
			Array(const this_type & prev) {
				/* Empty data */
				data = NULL;
				allocated = 0;
				frag = prev.frag;
				privatize(prev);
			}
			
			
			/** Desctructor
			 *
			 * Dispose the memory of the array.
			 *
			 */
			~Array() {
				if (data) /* Dispose data */
					delete[] data;
			}
			
			
			/** Copy assignment
			 *
			 * Copy array items from another array
			 * (copy is bit-by-bit).
			 *
			 * @param prev Array to copy.
			 *
			 */
			const this_type & operator=(const this_type & prev) {
				if (&prev != this) { /* No self-assignment */
					frag = prev.frag;
					privatize(prev);
				}
				return *this;
			}
			
			
			/** Implicit conversion operator
			 *
			 * Returns constant pointer to the array.
			 *
			 * @return Constant pointer.
			 *
			 */
			operator const Tmemb * () const {
				return (const Tmemb * ) data;
			}
			
			
			/** Append item at the end.
			 *
			 * Append one item at the end
			 * of the array.
			 *
			 * @param prev Member to append.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator<<(const Tmemb & prev) {
				allocate(len + 1);
				memcpy(data + len - 1, &prev, sizeof(Tmemb));
				
				return *this;
			}
			
			
			/** Append item at the beginning.
			 *
			 * Append item at the beginning
			 * of the array.
			 *
			 * Note: The appended item should
			 * not be a reference to another item
			 * from this array (in this case
			 * a wrong item will be appended).
			 *
			 * @param prev Item to append.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator>>(const Tmemb & prev) {
				allocate(len + 1);
				
				/* Shift original items
				   and put new item at the beginning */
				memmove(data + 1, data, len - 1);
				memcpy(data, &prev, sizeof(Tmemb));
				
				return *this;
			}
			
			
			/** Enlarge array by a number of items.
			 *
			 * Enlarge the array by allocating memory
			 * for 'nmemb' items at the end. The contents
			 * of the items is undefined.
			 *
			 * @param nmemb Number of items to allocate.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator+=(const size_type nmemb) {
				allocate(len + nmemb);
				return *this;
			}
			
			
			/** Shrink array by a number of items.
			 *
			 * Shrink the array by disposing 'nmemb'
			 * items at the end. If a larger count
			 * of items than the length of the array
			 * is specified, the array made empty.
			 *
			 * @param nmemb Number of items to dispose.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator-=(const size_type nmemb) {
				if (nmemb > len)
					allocate(0);
				else
					allocate(len - nmemb);
				
				return *this;
			}
			
			
			/** Get item reference.
			 *
			 * Get reference to an item from the array
			 * indexed by 'pos'. If 'pos' is out
			 * of range, ExceptionRange is thrown.
			 *
			 * @param pos Index of the member.
			 *
			 * @return Reference to a member.
			 *
			 */
			Tmemb & operator[](const size_type pos) const {
				CB2_RANGECHECK(pos < len, pos);
				return data[pos];
			}
			
			
			/** Copy items from C-style array.
			 *
			 * Copy items from a standard C-style
			 * array. Members are copied bit-by-bit.
			 *
			 * Original contents of the array is discarted.
			 *
			 * @param buffer Array with members to copy.
			 * @param nmemb  Number of members to copy.
			 *
			 */
			void Fill(const Tmemb * buffer, const size_type nmemb) {
				allocate(nmemb);
				memmove(data, buffer, sizeof(Tmemb) * nmemb);
			}
			
			
			/** Discart the array.
			 *
			 * Discart the contents of the array.
			 *
			 * @param shrink Whether the memory should
			 *               be disposed.
			 *
			 */
			void Discart(const bool shrink = false) {
				if (shrink) {
					/* Dispose data */
					if (data)
						delete[] data;
					data = NULL;
					allocated = 0;
				}
				allocate(0);
			}
			
			
			/** Return number of items.
			 *
			 * @return Number of items in the array.
			 *
			 */
			size_type Len() const {
				return len;
			}
			
			
			/** Check array for empty.
			 *
			 * @return True if the array is empty.
			 *
			 */
			bool Empty() const {
				return (len == 0);
			}
			
			
			/** Get starting reference
			 *
			 * Get the reference to the first
			 * item in the array as a pointer
			 * to the first item (or NULL on empty
			 * array).
			 *
			 * @return Starting reference.
			 *
			 */
			ref_type RefBegin() const {
				if (Len() > 0)
					return data;
					
				return NULL;
			}
			
			
			/** Get final reference
			 *
			 * Get the final reference as the pointer
			 * to the memory position after the last
			 * item in the array (or NULL on empty
			 * array).
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RefEnd() const {
				size_type len = Len();
				
				if (len > 0)
					return (data + len);
					
				return NULL;
			}
			
			
			/** Get starting reverse reference
			 *
			 * Get the reference to the last
			 * item in the array as a pointer
			 * RefEnd().
			 *
			 * @return Starting reverse reference.
			 *
			 */
			ref_type RevRefBegin() const {
				return RefEnd();
			}
	
			
			/** Get final reverse reference
			 *
			 * Get the final reverse reference,
			 * which is equivaltent to the starting
			 * forward reference.
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RevRefEnd() const {
				return RefBegin();
			}
			
			
			/** Get succeding reference
			 *
			 * Get the reference to an item
			 * succeding the item referenced
			 * by 'prev' (i.e. increase the pointer
			 * by one item). If the pointer is
			 * final or invalid, final reference
			 * is returned).
			 *
			 * @param prev Original reference.
			 *
			 * @return Next reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				if ((prev >= RefEnd()) || (prev < RefBegin())) /* Check pointer range */
					return RefEnd();
				
				return (prev + 1);
			}
			
			
			/** Get predecessing reference
			 *
			 * Get the reference to an item
			 * predecessing the item referenced
			 * by 'prev' (i.e. decrease the pointer
			 * by one item). If the pointer is
			 * final or invalid, final reference
			 * is returned).
			 *
			 * @param prev Original reference.
			 *
			 * @return Previous reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				if ((prev <= RevRefEnd()) || (prev > RevRefBegin())) /* Check pointer range */
					return RevRefEnd();
				
				return (prev - 1);
			}
			
			
			/** Dereference
			 *
			 * Get the item value referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const Tmemb & Deref(const ref_type & ref) const {
				/* Check pointer range */
				CB2_MEMORYCHECK((ref < RefEnd()) && (ref >= RefBegin()));
				return (*ref);
			}
			
			
			/** TODO
			 *
			 */
			const Tmemb & RevDeref(const ref_type & ref) const {
				/* Check pointer range */
				CB2_MEMORYCHECK((ref > RevRefEnd()) && (ref <= RevRefBegin()));
				return (*(ref - 1));
			}
			
			
			/** Get starting iterator
			 *
			 * @return Starting iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting reverse iterator
			 *
			 * @return Starting reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final reverse iterator
			 *
			 * @return Final reverse iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
			
			
			/** Cut out piece of array
			 *
			 * Remove several items from the array.
			 * The items are defined by the position of
			 * the first removed item and count of items
			 * that follows (and should be removed).
			 *
			 * If 'pos' or 'cnt' is out of range,
			 * ExceptionRange is thrown.
			 *
			 * @param pos Index of the first item to remove.
			 * @param cnt Number of items to remove.
			 *
			 */
			void Cut(const size_type pos, const size_type cnt) {
				/* Range check */
				CB2_RANGECHECK(pos < len, pos);
				CB2_RANGECHECK(pos + cnt <= len, pos);
				
				if (cnt > 0) {
					size_type mv = len - pos - cnt;
					
					if (mv > 0) /* Shift items front */
						memmove(data + pos, data + pos + cnt, sizeof(Tmemb) * mv);
					
					/* Discart overlapping items */
					allocate(len - cnt);
				}
			}
			
			
			/** Insert items
			 *
			 * Insert items from another array
			 * into this array. The first item from
			 * 'ins' is inserted at index 'pos'
			 * and totally 'cnt' items are inserted
			 * (previous items on these positions
			 * are shifted back).
			 *
			 * If 'pos' or 'cnt' is out of range,
			 * ExceptionRange is thrown.
			 *
			 * NB: This method doesn't support
			 * self-reference. If 'ins' is this
			 * same array, ExceptionAssert is thrown.
			 *
			 * @param pos Index of the first inserted item.
			 * @param cnt Number of items to insert.
			 * @param ins Array of items to insert.
			 *
			 */
			void Insert(const size_type pos, const size_type cnt, const this_type & ins) {
				CB2_ASSERT(&ins != this, "Cannot insert self-referred items");
			
				/* Range check */
				CB2_RANGECHECK(pos < len, pos);
				CB2_RANGECHECK(cnt < ins.Len(), cnt);
				
				size_t mv = len - pos;

				allocate(len + cnt);
				if (mv > 0) /* Shift items back */
					memmove(data + pos + cnt, data + pos, sizeof(Tmemb) * mv);
				
				/* Insert items */
				memcpy(data + pos, ins.data, sizeof(Tmemb) * cnt);
			}
			
		private:

			Tmemb * data;        /**< Data storage */
			size_type allocated; /**< Allocated memory space */
			size_type len;       /**< Members used */
			size_type frag;      /**< Fragment size */
			
			
			/** Allocate memory for items
			 *
			 * Allocate or reallocate memory
			 * for items.
			 *
			 * The memory is rounded to nearest
			 * fragment. If the new count of items
			 * is lower then the original count,
			 * the memory pool is untouched.
			 *
			 * If the allocation/reallocation
			 * fails, ExceptionMemory is thrown.
			 *
			 * @param nlen Number of items.
			 *
			 */
			void allocate(const size_type nlen) {
				/* Round items to nearest fragment */
				len = nlen;
				size_type nalloc = ((len / frag) + 1) * frag;
					
				if (nalloc > allocated) { /* Enlarge memory pool */
					Tmemb * newdata = new Tmemb[nalloc];
					if (data != NULL) {
						memcpy(newdata, data, sizeof(Tmemb) * allocated);
						delete[] data;
					}
					data = newdata;
					allocated = nalloc;
				}
			}
			
			
			/** Copy data from another array.
			 *
			 * Copy data from another array, unless
			 * this is self-reference.
			 *
			 * @param prev Array to copy from.
			 *
			 */
			void privatize(const this_type & prev) {
				if (&prev != this) {
					allocate(prev.len);
					memcpy(data, prev.data, sizeof(Tmemb) * prev.len);
				}
			}
			
	};
	
	
	template< class Tchar> class PString; /**< Forward declaration */
	
	
	/** Binary concatenation operator
	 *
	 * Concatenate two strings and get
	 * the output string.
	 *
	 * @param left  Left operand.
	 * @param right Right operand.
	 *
	 * @return Concatenated strign.
	 *
	 */
	template< class Tchar> PString< Tchar> operator+(const PString< Tchar> & left, const PString< Tchar> & right) {
		PString< Tchar> output(left);
		output += right;
		return output;
	}
	
	
	/** PString basic_ostream output operator
	 *
	 * Outpus the contents of a PString object
	 * to an output stream using the same
	 * basic type (Tchar).
	 *
	 */
	template< class Tchar> std::basic_ostream< Tchar, std::char_traits< Tchar> > & operator<<(std::basic_ostream< Tchar, std::char_traits< Tchar> > & stream, const PString< Tchar> & str) {
		stream.write(str.data, str.Len());
		return stream;
	}
	
	
	/** String template
	 *
	 * Template for representing character
	 * strings. Supports many features not
	 * even available by STL's string template.
	 *
	 * NB: The character (Tchar) type should
	 * be "primitive" (i.e. char, int, wchar_t, etc.),
	 * because Array< Tchar> is used as underlying
	 * data storage (which stores characters
	 * as bit copies).
	 *
	 */
	template< class Tchar> class PString : public Class {
		public:
			
			typedef PString< Tchar> this_type;                             /**< Template type alias */
			typedef Array< Tchar> storage_type;                            /**< Data storage type */
			typedef typename storage_type::size_type size_type;            /**< Size type */
			typedef typename storage_type::ref_type ref_type;              /**< Item reference type */
			typedef Iterator< this_type, ref_type, Tchar> iterator;        /**< Iterator */
			typedef RevIterator< this_type, ref_type, Tchar> rev_iterator; /**< Iterator */
			
			
			/** Default constructor
			 *
			 * Create an empty string.
			 *
			 */
			PString()
				: data(def_frag) {
				/* Add NULL-termination */
				data << 0;
			}
			
			
			/** Copy constructors
			 *
			 * Copy string data from another string
			 * repersentation.
			 *
			 * @param prev String to copy.
			 * @param dt   Raw character data.
			 * @param ln   Length of raw character data.
			 *
			 */
			PString(const this_type & prev) /**< From PString */
				: data(prev.data) {
			}
			
			PString(const Tchar * dt, const size_type ln) /**< From raw character data */
				: data(def_frag) {
				data.Fill(dt, ln);
				/* Add NULL-termination */
				data << 0;
			}
			
			PString(const Tchar * prev) /**< From NULL-terminated string */
				: data(def_frag) {
				cpy(prev);
			}
			
			PString(const std::basic_string< Tchar> & prev) /**< From std::basic_string< Tchar> */
				: data(def_frag) {
				cpy(prev);
			}
			
			PString(const Tchar prev) /**< From single Tchar */
				: data(def_frag) {
				cpy(prev);
			}
			
			PString(const signed int prev, const size_type base = 10) /**< From signed int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}
			
			PString(const unsigned int prev, const size_type base = 10)  /**< From unsigned int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}
			
			PString(const signed long int prev, const size_type base = 10)  /**< From signed long int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}

			PString(const unsigned long int prev, const size_type base = 10)  /**< From unsigned long int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}

			PString(const signed long long int prev, const size_type base = 10)  /**< From signed long long int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}

			PString(const unsigned long long int prev, const size_type base = 10)  /**< From unsigned long long int */
				: data(def_frag) {
				cpy_numeric_int(prev, base);
			}
			
			PString(const float prev) /**< From float */
				: data(def_frag) {
				cpy_numeric_float(prev);
			}
			
			PString(const double prev) /**< From double */
				: data(def_frag) {
				cpy_numeric_float(prev);
			}
			
			
			/** Assignment operator
			 *
			 * Copy string data from another string
			 * repersentation.
			 *
			 * @param prev String to copy.
			 *
			 * @return Reference to the result.
			 *
			 */
			const this_type & operator=(const this_type & prev) { /**< From PString */
				if (&prev != this)
					data = prev.data;
				return *this;
			}
			
			const this_type & operator=(const Tchar * prev) { /**< From NULL-terminated string */
				cpy(prev);
				return *this;
			}
			
			const this_type & operator=(const std::basic_string< Tchar> prev) { /**< From std::basic_string< Tchar> */
				cpy(prev);
				return *this;
			}
			
			const this_type & operator=(const Tchar prev) { /**< From single Tchar */
				cpy(prev);
				return *this;
			}
			
			const this_type & operator=(const signed int prev) { /**< From signed int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const unsigned int prev) {  /**< From unsigned int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const signed long int prev) {  /**< From signed long int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const unsigned long int prev) {  /**< From unsigned long int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const signed long long int prev) {  /**< From signed long long int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const unsigned long long int prev) {  /**< From unsigned long long int */
				cpy_numeric_int(prev, 10);
				return *this;
			}
			
			const this_type & operator=(const float prev) {  /**< From float */
				cpy_numeric_float(prev);
				return *this;
			}
			
			const this_type & operator=(const double prev) {  /**< From double */
				cpy_numeric_float(prev);
				return *this;
			}
			
			
			/** Append a character.
			 *
			 * Append a single character to the end
			 * of the string.
			 *
			 */
			const this_type & operator<<(const Tchar & right) {
				data[data.Len() - 1] = right;

				/* Add NULL-termination */
				data << 0;
				
				return *this;
			}
			
			
			/** Concatenation operators
			 *
			 * Unary and binary concatenation
			 * operators
			 *
			 * @param right Right concatenation operand.
			 * @param left  Left concatenation operand.
			 *
			 * @return Reference to the result.
			 *
			 */
			const this_type & operator+=(const this_type & right) { /**< Unary concatenation (+=) */
				/* Remove NULL-termination from left operand */
				data -= 1;
				
				/* Concatenate characters */
				for (size_type i = 0; i < right.Len(); i++)
					data << right.data[i];
				
				/* Add NULL-termination */
				data << 0;
				
				return *this;
			}
			
			const this_type & operator+(const this_type & right) { /**< Binary concatenation (+) */
				return operator+=(right);
			}
			
			friend PString< Tchar> operator+<>(const PString< Tchar> & left, const PString< Tchar> & right); /**< Binary concatenation operator (friend variant) */
			
			
			friend std::basic_ostream< Tchar, std::char_traits< Tchar> > & operator<<<>(std::basic_ostream< Tchar,  std::char_traits< Tchar> > & stream, const PString< Tchar> & str); /**< Stream output friend operator */
			
			
			/** Implicit conversion operators
			 *
			 * These operators do the implicit
			 * conversion to various data types.
			 *
			 * If the destination type cannot
			 * represent the current string value,
			 * ExceptionConversion is thrown.
			 *
			 * If the range of the destination type
			 * is not sufficient, ExceptionRange
			 * is thrown.
			 *
			 * @return String representation in given type.
			 *
			 */
			operator const Tchar * () const { /**< To NULL-terminated string */
				return (const Tchar * ) data;
			}
			
			operator std::basic_string< Tchar> () const { /**< To std::basic_string< Tchar> */
				std::basic_string< Tchar> str;
			
				for (size_type i = 0; i < Len(); i++)
					str += data[i];
					
				return str;
			}
			
			operator Tchar () const { /**< To single Tchar (first character) */
				CB2_CONVERSION(Len() > 0, "Can't convert empty string to character");
				return (Tchar) data[(size_type) 0];
			}
			
			operator signed int () const { /**< To signed int */
				return (signed int) numeric_int(true, (signed int) 0);
			}
			
			operator unsigned int () const {  /**< To unsigned int */
				return (unsigned int) numeric_int(false, (unsigned int) 0);
			}
			
			operator signed long int () const {  /**< To signed long int */
				return (signed long int) numeric_int(true, (signed long int) 0);
			}
			
			operator unsigned long int () const {  /**< To unsigned long int */
				return (unsigned long int) numeric_int(false, (unsigned long int) 0);
			}
			
			operator signed long long int () const {  /**< To signed long long int */
				return (signed long long int) numeric_int(true, (signed long long int) 0);
			}
			
			operator unsigned long long int () const {  /**< To unsigned long long int */
				return (unsigned long long int) numeric_int(false, (unsigned long long int) 0);
			}
			
			operator float () const { /**< To float */
				return (float) numeric_float((float) 0);
			}

			operator double () const { /**< To double */
				return (double) numeric_float((double) 0);
			}
			
			
			/** Number of Tchar elements
			 *
			 * @return Length of the string.
			 *
			 */
			size_type Len() const {
				size_type len = data.Len();
				CB2_ASSERT(len > 0, "Incosistent NULL-termination");
				return (len - 1);
			}
			
			
			/** Real size of the string in bytes
			 *
			 * @return Size of bytes representing
			 *         the string.
			 *
			 */
			size_type Size() const {
				return (Len() * sizeof(Tchar));
			}

			
			/** Indexaation operator
			 *
			 * Access to a random character
			 * of the string. If out of range,
			 * ExceptionRange is thrown.
			 *
			 * @param pos Index to access.
			 *
			 * @return Reference to the character
			 *         on the given position.
			 *
			 */
			Tchar & operator[](const size_type pos) const {
				CB2_RANGECHECK(pos < Len(), pos);
				return data[pos];
			}
			
			
			/** Compare strings
			 *
			 * Compare string contents with another
			 * string.
			 *
			 * @param str String to compare with.
			 *
			 * @return Compare result.
			 *
			 */
			bool operator==(const this_type & str) const { /**< With PString< Tchar> */
				if (Len() != str.Len())
					return false;
			
				for (size_type i = 0; i < Len(); i++) { /* Compare string characters */
					if (data[i] != str.data[i])
						return false;
				}
				
				return true;
			}
			
			
			bool operator!=(const this_type & str) const {
				return (!operator==(str));
			}
			
			bool operator==(const Tchar * str) const { /**< With Tchar * */
				if (Len() != strlen(str))
					return false;
			
				for (size_type i = 0; i < Len(); i++) { /* Compare string characters */
					if (data[i] != str[i])
						return false;
				}
				
				return true;
			}
			
			bool operator!=(const Tchar * str) const {
				return (!operator==(str));
			}

			
			/** Get substring
			 *
			 * Get substring from the current string.
			 * The substring is defined by the index
			 * of the first character and length.
			 *
			 * If 'base' is out of range or the string
			 * is empty, an empty string is returned.
			 * If 'length' is too long (or contains special
			 * value (size_type) -1, which is default),
			 * it is truncated to the maximal possible length.
			 *
			 * @param base   Index of first character.
			 * @param length Number of characters to copy.
			 *
			 * @return Substring.
			 *
			 */
			this_type Substr(const size_type base, const size_type length = (size_type) -1) const {
				if ((Empty()) || (base >= Len()))
					return this_type();
				
				size_type len;
				
				if ((length == (size_type) -1) || (base + length > Len())) /* Truncate length */
					len = Len() - base;
				else
					len = length;
				
				return this_type(data + base, len);
			}
			
			
			/** Check for empty
			 *
			 * @return True if string is empty.
			 *
			 */
			bool Empty() const {
				return (Len() == 0);
			}
			
			
			/** Discart the string.
			 *
			 * Discart the contents of the string.
			 *
			 * @param shrink Whether the memory should
			 *               be disposed.
			 *
			 */
			void Discart(const bool shrink = false) {
				data.Discart(shrink);
				/* Add NULL-termination */
				data << 0;
			}
			
			
			/** Find position of substring
			 *
			 * Get the position of the nearest
			 * appearance of 'needle' beginning
			 * at index 'from'.
			 *
			 * If 'needle' or this string are empty,
			 * false is returned.
			 *
			 * @param needle Substring to search.
			 * @param pos    Position of the appearance.
			 * @param from   Index to search from.
			 *
			 * @return True if substring is found.
			 *
			 */
			bool Pos(const this_type & needle, size_type & pos, const size_type from = 0) const {
				if ((Empty()) || (needle.Empty()))
					return false;
			
				size_type ni = 0;
				
				for (size_type i = from; i < Len(); i++) { /* For each character in haystack */
					while ((i + ni < Len()) && (data[i + ni] == needle.data[ni])) { /* Compare characters with needle */
						ni++;
						
						if (ni == needle.Len()) { /* Found match */
							pos = i;
							return true;
						}
					}
					ni = 0;
				}
				
				return false;
			}
			
			/** Substitute substring
			 *
			 * Substitute all appearances of 'needle'
			 * by 'replacement'.
			 *
			 * @param needle      String to search.
			 * @param replacement String to substitute.
			 *
			 * @return Number of substitutes done.
			 *
			 */
			size_type Subst(const this_type & needle, const this_type & replacement) {
				size_type cnt = 0;
				size_type i = 0;
				size_type pos;
				size_type nlen = needle.Len();
				size_type rlen = replacement.Len();
				
				while (Pos(needle, pos, i)) {
					data.Cut(pos, nlen);                      /* Cut out needle */
					data.Insert(pos, rlen, replacement.data); /* Insert replacement */
					
					i = pos + rlen;
					cnt++;
				}
				
				return cnt;
			}
			
			
			/** Escape HTML special characters
			 *
			 * Escape special HTML characters
			 * by its &entity; equivalents. Currently
			 * &, <, > and " are handled.
			 *
			 * @param nl If non-empty, all newlines
			 *           are substituted by this string.
			 *
			 * @return Escaped string.
			 *
			 */
			this_type HTML(const this_type & nl = "") const {
				this_type str;
				bool nl_subst = (nl != "");
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case '&':
							str += "&amp;";
							break;
						case '<':
							str += "&lt;";
							break;
						case '>':
							str += "&gt;";
							break;
						case '"':
							str += "&quot;";
							break;
						case '\n':
							if (nl_subst) {
								str += nl;
								break;
							}
						default:
							str << data[i];
					}
				}
				
				return str;
			}
			
			
			/** Decode HTML special characters
			 *
			 * Currently &amp;, &lt;, &gt; and &quot;
			 * are interpreted.
			 *
			 * @return Decoded string.
			 *
			 */
			this_type HTMLdecode() const {
				this_type str;
				
				for (size_type i = 0; i < Len(); i++) {
					if (data[i] == '&') {
						if (Substr(i + 1, 4) == "amp;") {
							str << '&';
							i += 4;
						} else if (Substr(i + 1, 3) == "lt;") {
							str << '<';
							i += 3;
						} else if (Substr(i + 1, 3) == "gt;") {
							str << '>';
							i += 3;
						} else if (Substr(i + 1, 5) == "quot;") {
							str << '"';
							i += 5;
						} else
							CB2_CONVERSION(false, "Unsupported HTML entity");
					} else
						str << data[i];
				}
				
				return str;
			}
			
			
			/** Escape SQL special characters
			 *
			 * Escape special SQL characters
			 * by its \symbol equivalents. Currently
			 * ', ", \ and NULL are handled.
			 *
			 * @return Escaped string.
			 *
			 */
			this_type SQL() const {
				this_type str;
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case '\'':
							str += "\\\'";
							break;
						case '"':
							str += "\\\"";
							break;
						case '\\':
							str += "\\\\";
							break;
						case 0:
							str += "\\0";
							break;
						default:
							str << data[i];
					}
				}
				
				return str;
			}
			
			
			/** Decode SQL special characters
			 *
			 * Simply all characters escaped as
			 * \symbol are decoded as given symbol
			 * (except \0, which is interpreted
			 * as NULL character).
			 *
			 * @return Escaped string.
			 *
			 */
			this_type SQLdecode() const {
				this_type str;
				
				for (size_type i = 0; i < Len(); i++) {
					if (data[i] == '\\') {
						CB2_CONVERSION(i + 1 < Len(), "Illegal character escape");
						if (data[i + 1] == '0')
							str << (Tchar) 0;
						else
							str << data[i + 1];
						i++;
					} else
						str << data[i];
				}
				
				return str;
			}
			
			
			/** Escape URL special characters
			 *
			 * Escape all non-alphanumeric characters
			 * as %xx (where xx is the hexadecimal
			 * representation of the character) as defined
			 * by RFC1738, but leave '-' and '_' untouched
			 * and substitute space by '+'.
			 *
			 * @return Escaped string.
			 *
			 */
			this_type URL() const {
				this_type str;
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case 'a'...'z':
						case 'A'...'Z':
						case '0'...'9':
						case '-':
						case '_':
							str << data[i];
							break;
						case ' ':
							str << '+';
							break;
						default:
							str << '%';
							
							int ord = (((int) data[i]) >> 4) & 15;
							if (ord < 10)
								str << (Tchar) ((int) '0' + ord);
							else
								str << (Tchar) ((int) 'A' + ord);
							
							ord = ((int) data[i]) & 15;
							if (ord < 10)
								str << (Tchar) ((int) '0' + ord);
							else
								str << (Tchar) ((int) 'A' + ord);
					}
				}
				
				return str;
			}
			
			/** Decode URL special characters
			 *
			 * Decode string according to RFC1738
			 * (interpreting %xx as hexadecimal representation
			 * of characters). Plus interpreting '+'
			 * as space.
			 *
			 * @return Escaped string.
			 *
			 */
			this_type URLdecode() const {
				this_type str;
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case '+':
							str << ' ';
							break;
						case '%':
							CB2_CONVERSION(i + 2 < Len(), "Illegal character escape");
							
							Tchar c;
							
							switch (data[i + 1]) {
								case '0'...'9':
									c = (data[i + 1] - '0') << 4;
									break;
								case 'a'...'f':
									c = (data[i + 1] - 'a' + 10) << 4;
									break;
								case 'A'...'F':
									c = (data[i + 1] - 'A' + 10) << 4;
									break;
								default:
									CB2_CONVERSION(false, "Illegal hexadecimal character");
							}
							
							switch (data[i + 2]) {
								case '0'...'9':
									c += (data[i + 2] - '0');
									break;
								case 'a'...'f':
									c += (data[i + 2] - 'a' + 10);
									break;
								case 'A'...'F':
									c += (data[i + 2] - 'A' + 10);
									break;
								default:
									CB2_CONVERSION(false, "Illegal hexadecimal character");
							}
							str << c;
							i += 2;
							break;
						default:
							str << data[i];
					}
				}
				
				return str;
			}
			
			
			/** Get starting reference
			 *
			 * Get the reference to the first
			 * character in the string
			 * (as a reference to the array).
			 *
			 * @return Starting reference.
			 *
			 */
			ref_type RefBegin() const {
				return data.RefBegin();
			}
			
			
			/** Get final reference
			 *
			 * Get the final reference. This is
			 * the reference to the last character
			 * in the array (NULL-termination).
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RefEnd() const {
				return data.RefPrev(data.RevRefBegin());
			}
			
			
			/** Get starting reverse reference
			 *
			 * Get the reference to the last
			 * character.
			 *
			 * @return Starting reverse reference.
			 *
			 */
			ref_type RevRefBegin() const {
				return data.RefPrev(data.RevRefBegin());
			}
			
			
			/** Get final reverse reference
			 *
			 * Get the reference to the first
			 * character.
			 * 
			 * @return Final reverse reference.
			 *
			 */
			ref_type RevRefEnd() const {
				return data.RevRefEnd();
			}
			
			
			/** Get succeding reference
			 *
			 * Get the reference to a character
			 * succeding the character referenced
			 * by 'prev'. If the reference is
			 * final or invalid, final reference
			 * is returned).
			 *
			 * @param prev Original reference.
			 *
			 * @return Next reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				if ((prev >= RefEnd()) || (prev < RefBegin())) /* Check ranges */
					return RefEnd();
				
				return data.RefNext(prev);
			}
			
			
			/** Get predecessing reference
			 *
			 * Get the reference to a character
			 * predecessing the character referenced
			 * by 'prev'. If the reference is
			 * final or invalid, final reference
			 * is returned).
			 *
			 * @param prev Original reference.
			 *
			 * @return Previous reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				if ((prev <= RevRefEnd()) || (prev > RevRefBegin())) /* Check ranges */
					return RevRefEnd();
				
				return data.RefPrev(prev);
			}
			
			
			/** Dereference
			 *
			 * Get the character referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const Tchar & Deref(const ref_type & ref) const {
				/* Check pointer range */
				CB2_MEMORYCHECK((ref < RefEnd()) && (ref >= RefBegin()));
				return (*ref);
			}
			
			
			/** Reverse dereference
			 *
			 * Get the character referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const Tchar & RevDeref(const ref_type & ref) const {
				/* Check pointer range */
				CB2_MEMORYCHECK((ref > RevRefEnd()) && (ref <= RevRefBegin()));
				return (*(ref - 1));
			}
			
			
			/** Get starting iterator
			 *
			 * @return Starting iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting reverse iterator
			 *
			 * @return Starting reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final reverse iterator
			 *
			 * @return Final reverse iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
			
		private:
		
			storage_type data ;                     /**< Data storage */
			static const size_type def_frag = 1024; /**< Default array fragment size */

			
			/** Copy string
			 *
			 * Do the conversion from another
			 * data type.
			 *
			 * @param prev Data to convert from.
			 *
			 */
			void cpy(const Tchar * prev) { /**< From NULL-terminated string */
				data.Fill(prev, strlen(prev));
				data << 0;
			}
			
			void cpy(const std::basic_string< Tchar> & prev) { /**< From std::basic_string< Tchar> */
				data.Discart();
				
				for (typename std::basic_string< Tchar>::size_type i = 0; i < prev.length(); i++)
					data << prev[i];
				
				data << 0;
			}
			
			void cpy(const Tchar prev) { /**< From single Tchar */
				data.Fill(&prev, 1);
				data << 0;
			}

			
			/** Copy string from integer numeric data type
			 *
			 * Do the conversion from integer numeric
			 * data type.
			 *
			 * NB: The numeric base is not stored
			 * in the string
			 *
			 * @param prev Numeric value to convert from.
			 * @param base Numeric base.
			 *
			 */
			template < class Tnum> void cpy_numeric_int(const Tnum prev, const size_type base) {
				CB2_ASSERT((base > 1) || (base < 37), "Invalid numeric base");
			
				bool sign;
				Tnum value = prev;
				
				data.Discart();
				
				if (value < 0) { /* Work with absolute value, note sign */
					sign = true;
					value = -value;
				} else
					sign = false;
				
				while (value > 0) { /* Get orders */
					Tnum ord = value % base;
					value = value / base;
					if (ord < 10)
						data >> ((Tnum) '0' + ord);
					else
						data >> ((Tnum) 'A' + ord);
				}
				
				if (sign) /* Add unary minus */
					data >> '-';
				
				data << 0;
			}
			
			
			/** Copy string from floating point numeric data type
			 *
			 * Do the conversion from floating
			 * point numeric data type.
			 *
			 * @param prev Numeric value to convert from.
			 *
			 */
			template < class Tnum> void cpy_numeric_float(const Tnum prev) {
				bool sign;
				Tnum value = prev;
				
				data.Discart();
				
				if (value < 0) { /* Work with absolute value, note sign */
					sign = true;
					value = -value;
				} else
					sign = false;
					
				Tnum value_floor = floor(prev);
				Tnum value_diff = value_floor - value;
				
				while (value_floor > 0) { /* Itegral part */
					Tnum oval = value_floor;
					value_floor = floor(value_floor / 10);
					int ord = (int) (oval - value_floor * 10);
					data >> ((int) '0' + ord);
				}
				
				if (sign)
					data >> '-';
				
				bool first = true;
				while (value_diff > 0) { /* Fraction part */
					value_diff = value_diff * 10;
					int ord = (int) value_diff;
					value_floor = floor(value_diff);
					value_diff = value_diff - value_floor;
					if (first) {
						data << '.';
						first = false;
					}
					data << ((int) '0' + ord);
				}
				
				data << 0;
			}
			
			
			/** Convert to integer data type
			 *
			 * Convert current string value
			 * to numeric data type.
			 *
			 * @param signed_type Is the numeric type signed?
			 * @param zero        Zero value of the numeric type.
			 *
			 * @return Requested value.
			 *
			 */
			template< class Tnum> Tnum numeric_int(const bool signed_type, const Tnum zero) const {
				Tnum value = zero;
				Tnum base = 10;
				Tnum literal;
				
				int state;
				/* State numbers:
				    0 ... awaiting sign character, base prefix or literal
					1 ... awaiting base prefix or literal
					2 ... awaiting literal
					3 ... tailing whitespace */
				bool sign = false;
				
				if (signed_type)
					state = 0;
				else
					state = 1; /* Unsigned type -> don't accept sign character */
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case '+': /* Unary plus */
							switch (state) {
								case 0:
									state = 1;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced sign characters");
							}
							break;
						case '-': /* Unary minus */
							if (!signed_type)
								CB2_CONVERSION(false, "Unary minus while converting to unsigned type");
							switch (state) {
								case 0:
									state = 1;
									sign = true;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced sign characters");
							}
							break;
						case '0': /* '0' as literal or as base prefix */
							if (state == 1) { 
								if ((i + 1 < Len()) && (data[i + 1] == 'x')) { /* Leading '0x' -> hexadecimal */
									base = 16;
									state = 2;
									i++;
									break;
								} else { /* Leading '0' -> octal */
									base = 8;
									state = 2;
									break;
								}
							}
						case '1'...'9': /* Literals */
							switch (state) {
								case 0:
								case 1:
								case 2:
									state = 2;
									literal = (Tnum) data[i] - (Tnum) '0';
									CB2_CONVERSION(literal < base, "Invalid numeric literal");
									value *= base;
									value += literal;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced character");
							}
							break;
						case 'a'...'z': /* Literals */
							switch (state) {
								case 0:
								case 1:
								case 2:
									state = 2;
									literal = (Tnum) data[i] - (Tnum) 'a' + 10;
									CB2_CONVERSION(literal < base, "Invalid numeric literal");
									value *= base;
									value += literal;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced character");
							}
							break;
						case 'A'...'Z': /* Literals */
							switch (state) {
								case 0:
								case 1:
								case 2:
									state = 2;
									literal = (Tnum) data[i] - (Tnum) 'A' + 10;
									CB2_CONVERSION(literal < base, "Invalid numeric literal");
									value *= base;
									value += literal;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced character");
							}
							break;
						case ' ':
						case '\t':
						case 0: /* Beginning or tailing whitespace */
							switch (state) {
								case 0:
									break;
								case 2:
									state = 3;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced whitespace");
							}
							break;
						default:
							CB2_CONVERSION(false, "Invalid character");
					}
				}
				
				if (sign) /* Sign */
					value = -value;
				
				CB2_CONVERSION(state >= 2, "Invalid number");
					
				return value;
				
			}
			
			
			/** Convert to floating point data type
			 *
			 * Convert current string value
			 * to floating point numeric data type.
			 *
			 * Just simple representations are accepted.
			 *
			 * @param signed_type Is the numeric type signed?
			 * @param zero        Zero value of the numeric type.
			 *
			 * @return Requested value.
			 *
			 */
			template< class Tnum> Tnum numeric_float(const Tnum zero) const {
				Tnum value = zero;
				Tnum fraction = 10;
				
				int state = 0;
				/* State numbers:
				    0 ... awaiting sign character, integer literal or period
					1 ... integer literal or period
					2 ... awaiting fraction literal
					3 ... tailing whitespace */
				bool sign = false;
				
				for (size_type i = 0; i < Len(); i++) {
					switch (data[i]) {
						case '+': /* Unary plus */
							switch (state) {
								case 0:
									state = 1;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced sign characters");
							}
							break;
						case '-': /* Unary minus */
							switch (state) {
								case 0:
									state = 1;
									sign = true;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced sign characters");
							}
							break;
						case '0'...'9': /* Literal */
							switch (state) {
								case 0:
								case 1: /* Integral part */
									state = 1;
									value *= 10;
									value += (Tnum) data[i] - (Tnum) '0';
									break;
								case 2: /* Fraction part */
									value += ((Tnum) data[i] - (Tnum) '0') / fraction;
									fraction *= 10;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced character");
							}
							break;
						case '.': /* Period */
							switch (state) {
								case 0:
								case 1:
									state = 2;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced period");
							}
							break;
						case ' ':
						case '\t':
						case 0: /* Beginning or tailing whitespace */
							switch (state) {
								case 0:
									break;
								case 1:
								case 2:
									state = 3;
									break;
								default:
									CB2_CONVERSION(false, "Misplaced whitespace");
							}
							break;
						default:
							CB2_CONVERSION(false, "Invalid character");
					}
				}
				
				if (sign) /* Sign */
					value = -value;
				
				CB2_CONVERSION(state >= 1, "Invalid number");
					
				return value;
				
			}
			
			
	};
	
	
	/** Generic list template
	 *
	 * This class stores items of the same
	 * type in a bi-directional pointer list.
	 * The items are stored either as copies
	 * or as references (pointers to the original
	 * objects).
	 *
	 * Unlike Array template, this class is object
	 * safe (for objects stored as local copies),
	 * but requies a public copy constructor to be present
	 * in the Tmemb class.
	 *
	 */
	template< class Tmemb> class List : public Class {
		public:
			
			typedef List< Tmemb> this_type; /**< Template type alias */
			typedef size_t size_type;       /**< Size type */
			
			/** Item type
			 *
			 * One item of the list, containing
			 * pointer to the data, flag whether
			 * the data is allocated by List or
			 * a foreign pointer and pointers
			 * to previous and next items in
			 * the list.
			 *
			 */
			typedef struct _item_type {
				Tmemb * data;             /**< Data */
				bool local;               /**< Local copy flag */
				struct _item_type * prev; /**< Previous item in list */
				struct _item_type * next; /**< Next item in list */
			} item_type;               
			
			typedef item_type * ref_type;                                  /**< Item reference type */
			typedef Iterator< this_type, ref_type, Tmemb> iterator;        /**< Iterator */
			typedef RevIterator< this_type, ref_type, Tmemb> rev_iterator; /**< Reverse iterator */
		

			/** Default constructor
			 *
			 * Create empty list.
			 *
			 */
			List() {
				list = NULL;
				tail = NULL;
				len = 0;
			}
			
			
			/** Copy constructor
			 *
			 * Copy list items from another list.
			 *
			 * @param prev List to copy from.
			 *
			 */
			List(const this_type & prev) {
				for (item_type * ac = prev.list; ac != NULL; ac = ac->next) {
					if (ac->local)
						PushBack(*(ac->data));
					else
						operator<<(ac->data);
				}
			}
			
			
			/** Desctructor
			 *
			 * Dispose the memory of the list.
			 *
			 */
			~List() {
				Discart();
			}
			
			
			/** Copy assignment
			 *
			 * Copy list items from another list.
			 *
			 * @param prev List to copy from.
			 *
			 */
			const this_type & operator=(const this_type & prev) {
				if (&prev != this) { /* No self-assignment */
					Discart();
					for (item_type * ac = prev.list; ac != NULL; ac = ac->next) { /* Copy list items */
						if (ac->local)
							PushBack(*(ac->data));
						else
							operator<<(ac->data);
					}
				}
				return *this;
			}
			
			
			/** Append item at the end.
			 *
			 * Append one item at the end
			 * of the list. Traditionally
			 * this operator stores just
			 * the pointer to the original
			 * instance of the item. If you
			 * want to make a local copy,
			 * use PushBack() method instead.
			 *
			 * @param prev Item to append.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator<<(Tmemb * prev) {
				if (list == NULL) {
					list = new item_type;
					list->data = prev;
					list->local = false;
					list->prev = NULL;
					list->next = NULL;
					tail = list;
				} else {
					tail->next = new item_type;
					tail->next->data = prev;
					tail->next->local = false;
					tail->next->prev = tail;
					tail->next->next = NULL;
					tail = tail->next;
				}
				len++;
				
				return *this;
			}
			
			
			/** Append item at the beginning.
			 *
			 * Append item at the beginning
			 * of the list. Traditionally
			 * this operator stores just
			 * the pointer to the original
			 * instance of the item. If you
			 * want to make a local copy,
			 * use PushFront() method instead.
			 *
			 * @param prev Item to append.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator>>(Tmemb * prev) {
				if (list == NULL) {
					list = new item_type;
					list->data = prev;
					list->local = false;
					list->prev = NULL;
					list->next = NULL;
					tail = list;
				} else {
					item_type * ac = new item_type;
					ac->data = prev;
					ac->local = false;
					ac->prev = NULL;
					ac->next = list;
					list->prev = ac;
					list = ac;
				}
				len++;
				
				return *this;
			}
			
			
			/** Append item at the end.
			 *
			 * Append one item at the end
			 * of the list. This method stores
			 * a local copy of the item,
			 * so a copy constructor is needed.
			 *
			 * @param prev Item to append.
			 *
			 */
			void PushBack(const Tmemb & prev) {
				Tmemb * local = new Tmemb(prev);
				
				if (list == NULL) {
					list = new item_type;
					list->data = local;
					list->local = true;
					list->prev = NULL;
					list->next = NULL;
					tail = list;
				} else {
					tail->next = new item_type;
					tail->next->data = local;
					tail->next->local = true;
					tail->next->prev = tail;
					tail->next->next = NULL;
					tail = tail->next;
				}
				len++;
			}
			
			
			/** Append item at the beginning.
			 *
			 * Append item at the beginning
			 * of the list. This method stores
			 * a local copy of the item,
			 * so a copy constructor is needed.
			 *
			 * @param prev Item to append.
			 *
			 */
			void PushFront(const Tmemb & prev) {
				Tmemb * local = new Tmemb(prev);
				
				if (list == NULL) {
					list = new item_type;
					list->data = local;
					list->local = true;
					list->prev = NULL;
					list->next = NULL;
					tail = list;
				} else {
					item_type * ac = new item_type;
					ac->data = local;
					ac->local = true;
					ac->prev = NULL;
					ac->next = list;
					list->prev = ac;
					list = ac;
				}
				len++;
			}
			
			
			/** Get item reference.
			 *
			 * Get reference to an item from the list
			 * indexed by 'pos'. If 'pos' is out
			 * of range, ExceptionRange is thrown.
			 *
			 * @param pos Index of the member.
			 *
			 * @return Reference to a member.
			 *
			 */
			Tmemb & operator[](const size_type pos) const {
				CB2_RANGECHECK(pos < len, pos);
				
				item_type * ac = list;
				size_type i = pos;
				
				while (ac != NULL) {
					if (i == 0)
						return *(ac->data);
					i--;
					ac = ac->next;
				}
				
				CB2_ASSERT(false, "Inconsistent list data");
				return *(list->data); /* Unreachable, just to supress compiler errors */
			}
			
			
			/** Discart the array.
			 *
			 * Discart the contents of the array.
			 *
			 */
			void Discart() {
				item_type * ac = list;
				
				while (ac != NULL) {
					if (ac->local)
						delete ac->data;
					
					item_type * next = ac->next;
					delete ac;
					ac = next;
				}
				
				list = NULL;
				tail = NULL;
			}
			
			
			/** Return number of items.
			 *
			 * @return Number of items in the array.
			 *
			 */
			size_type Len() const {
				return len;
			}
			
			
			/** Check array for empty.
			 *
			 * @return True if the array is empty.
			 *
			 */
			bool Empty() const {
				return (len == 0);
			}
			
			
			/** Get the first item
			 *
			 * @return First item in the list.
			 *
			 */
			Tmemb * Head() const {
				if (list != NULL)
					return list->data;
				
				return NULL;
			}
			
			
			/** Get the last item
			 *
			 * @return Last item in the list.
			 *
			 */
			Tmemb * Tail() const {
				if (tail != NULL)
					return tail->data;
				
				return NULL;
			}
			
			
			/** Get starting reference
			 *
			 * Get the reference to the first
			 * item in the list as a pointer
			 * to the first item (or NULL on empty
			 * list).
			 *
			 * @return Starting reference.
			 *
			 */
			ref_type RefBegin() const {
				return list;
			}
			
			
			/** Get final reference
			 *
			 * Get the final reference as a NULL pointer.
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RefEnd() const {
				return NULL;
			}
			
			
			/** Get starting reverse reference
			 *
			 * Get the reverse reference to the last
			 * item in the list as a pointer
			 * to the last item (or NULL on empty
			 * list).
			 *
			 * @return Starting reverse reference.
			 *
			 */
			ref_type RevRefBegin() const {
				return tail;
			}
			
			
			/** Get final reverse reference
			 *
			 * Get the final reverse reference as a NULL pointer.
			 *
			 * @return Final reverse reference.
			 *
			 */
			ref_type RevRefEnd() const {
				return NULL;
			}
			
			
			/** Get succeding reference
			 *
			 * Get the reference to an item
			 * succeding the item referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Next reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				return prev->next;
			}
			
			
			/** Get predecessing reference
			 *
			 * Get the reference to an item
			 * predecessing the item referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Previous reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				return prev->prev;
			}
			
			
			/** Dereference
			 *
			 * Get the item value referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const Tmemb & Deref(const ref_type & ref) const {
				CB2_MEMORYCHECK(ref != NULL);
				return *(ref->data);
			}
			
			
			/** Reverse dereference
			 *
			 * Get the item value referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const Tmemb & RevDeref(const ref_type & ref) const {
				CB2_MEMORYCHECK(ref != NULL);
				return *(ref->data);
			}
			
			
			/** Get starting iterator
			 *
			 * @return Starting iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting reverse iterator
			 *
			 * @return Starting reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final reverse iterator
			 *
			 * @return Final reverse iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
			
		private:

			item_type * list; /**< Data storage */
			item_type * tail; /**< Pointer to the tail of the list */
			size_type len;    /**< Members used */
			
	};
	
	
	/** Map key to value
	 *
	 * This template stores a tuple of values,
	 * where the first one is considered 'key'
	 * and the other (which might not be present,
	 * see the SQL 'NULL' semantics) is considered 'value'.
	 *
	 */
	template< class Tkey, class Tvalue> class PMap : public Class {
		public:
			
			typedef PMap< Tkey, Tvalue> this_type; /**< Template type alias */
			
			
			/** Constructor
			 *
			 * Assign key to value.
			 *
			 * @param key   Key.
			 * @param value Value.
			 *
			 */
			PMap(const Tkey & key, const Tvalue & value) {
				_key = key;
				_value = value;
				_null = false;
			}
			
			
			/** Constructor
			 *
			 * Assign a 'NULL' value to key.
			 *
			 * @param key Key.
			 *
			 */
			PMap(const Tkey & key) {
				_key = key;
				_null = true;
			}
			
			
			/** Copy constructor
			 *
			 * Copy key=value pair from another
			 * Map.
			 *
			 * @param prev Map to copy from.
			 *
			 */
			PMap(const this_type & prev) {
				_key = prev._key;
				if (prev._null) /* NULL value */
					_null = true;
				else { /* Non-NULL value */
					_value = prev._value;
					_null = false;
				}
			}
			
			
			/** Assignment operator
			 * Copy key=value pair from another
			 * Map.
			 *
			 * @param prev Map to copy from.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator=(const this_type & prev) {
				if (&prev != this) { /* No self-assignment */
					_key = prev._key;
					if (prev._null) /* NULL value */
						_null = true;
					else { /* Non-NULL value */
						_value = prev._value;
						_null = false;
					}
				}
				return *this;
			}
			
			
			/** Get key
			 *
			 * @return Key.
			 *
			 */
			const Tkey & GetKey() const {
				return _key;
			}
			
			
			/** Get value
			 *
			 * Get the value. If the value is NULL,
			 * ExceptionAssert is thrown.
			 *
			 * @return Value.
			 *
			 */
			const Tvalue & GetValue() const {
				CB2_ASSERT(_null == false, "Value of NULL field is undefined");
				return _value;
			}
			
			
			/** Check NULL
			 *
			 * @return True if the value is NULL.
			 *
			 */
			bool IsNULL() const {
				return _null;
			}
			
		private:
			
			Tkey _key;     /**< Data item key */
			Tvalue _value; /**< Data item value */
			bool _null;    /**< Value is NULL (SQL syntax) */
			
	};
	
	
	/** List of key=value pairs
	 *
	 * List of key=value pairs of a given type.
	 * Unlike the simple List< Map> type, this
	 * template allows access by key.
	 *
	 */
	template< class Tkey, class Tvalue> class PMapList : public Class {
		public:
			
			typedef PMapList< Tkey, Tvalue> this_type;                         /**< Template type alias */
			typedef PMap< Tkey, Tvalue> item_type;                             /**< Item type */
			typedef List< item_type> storage_type;                             /**< Storage type */
			typedef typename storage_type::size_type size_type;                /**< Size type */
			typedef typename storage_type::ref_type ref_type;                  /**< Item reference type */
			typedef Iterator< this_type, ref_type, item_type> iterator;        /**< Iterator */
			typedef RevIterator< this_type, ref_type, item_type> rev_iterator; /**< Reverse iterator */
			
			
			/** Constructor
			 *
			 * Create empty list.
			 *
			 */
			PMapList() {
				/* Empty */
			}
			
			
			/** Destructor
			 *
			 * Destroy the list.
			 *
			 */
			~PMapList() {
				/* Empty */
			}
			
			
			/** Add new pair
			 *
			 * Add new pair of key=value to the list.
			 * The pair is stored as local copy.
			 *
			 * @param map Pair to store.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator<<(const item_type & map) {
				data.PushBack(map);
				return *this;
			}
			
			
			/** Add new pair
			 *
			 * Add new pair as key and value.
			 *
			 * @param key   Key.
			 * @param value Value.
			 *
			 */
			void Add(const Tkey & key, const Tvalue & value) {
				item_type map(key, value);
				data.PushBack(map);
			}
			
			
			/** Add new pair with NULL value
			 *
			 * Add new pair as key and NULL value.
			 *
			 * @param key   Key.
			 *
			 */
			void Add(const Tkey & key) {
				item_type map(key);
				data.PushBack(map);
			}
			
			
			/** Access item by key
			 *
			 * Get value by key. If no value
			 * for given key is found, ExceptionNotFound
			 * is thown. Also if the value is found,
			 * but NULL, ExceptionAssert is thrown
			 * (see Map< Tkey, Tvalue>::GetValue().
			 *
			 * @param key Key.
			 *
			 * @return Value.
			 *
			 */
			const Tvalue & operator[](const Tkey & key) const {
				for (typename storage_type::iterator it = data.Begin(); it != data.End(); ++it) {
					if (it->GetKey() == key)
						return it->GetValue();
				}
				CB2_NOTFOUND(false);
			}
			
			
			/** Check whether item is NULL
			 *
			 * Check whether an item is NULL. If no value
			 * for given key is found, ExceptionNotFound
			 * is thrown.
			 *
			 * @param key Key.
			 *
			 * @return True if value for the key is NULL.
			 *
			 */
			bool IsNULL(const Tkey & key) const {
				for (typename storage_type::iterator it = data.Begin(); it != data.End(); ++it) {
					if (it->GetKey() == key)
						return it->IsNULL();
				}
				CB2_NOTFOUND(false);
			}
			
			
			/** Return number of items.
			 *
			 * @return Number of items in the list.
			 *
			 */
			size_type Len() const {
				return data.Len();
			}
			
			
			/** Check list for empty.
			 *
			 * @return True if the list is empty.
			 *
			 */
			bool Empty() const {
				return data.Empty();
			}
			
			
			/** Get starting reference
			 *
			 * Get the reference to the first
			 * item in the list as a pointer
			 * to the first item (or NULL on empty
			 * list).
			 *
			 * @return Starting reference.
			 *
			 */
			ref_type RefBegin() const {
				return data.RefBegin();
			}
			
			
			/** Get final reference
			 *
			 * Get the final reference as a NULL pointer.
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RefEnd() const {
				return data.RefEnd();
			}
			
			
			/** Get starting reverse reference
			 *
			 * Get the reverse reference to the last
			 * item in the list as a pointer
			 * to the last item (or NULL on empty
			 * list).
			 *
			 * @return Starting reverse reference.
			 *
			 */
			ref_type RevRefBegin() const {
				return data.RevRefBegin();
			}
			
			
			/** Get final reverse reference
			 *
			 * Get the final reverse reference as a NULL pointer.
			 *
			 * @return Final reverse reference.
			 *
			 */
			ref_type RevRefEnd() const {
				return data.RevRefEnd();
			}
			
			
			/** Get succeding reference
			 *
			 * Get the reference to an item
			 * succeding the item referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Next reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				return data.RefNext(prev);
			}
			
			
			/** Get predecessing reference
			 *
			 * Get the reference to an item
			 * predecessing the item referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Previous reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				return data.RefPrev(prev);
			}
			
			
			/** Dereference
			 *
			 * Get the item value referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const item_type & Deref(const ref_type & ref) const {
				return data.Deref(ref);
			}
			
			
			/** Reverse dereference
			 *
			 * Get the item value referenced by 'ref'.
			 * If the reference is out of range,
			 * ExceptionMemory is thrown.
			 *
			 * @param ref Item reference.
			 *
			 * @return Item value.
			 *
			 */
			const item_type & RevDeref(const ref_type & ref) const {
				return data.RevDeref(ref);
			}
			
			
			/** Get starting iterator
			 *
			 * @return Starting iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting reverse iterator
			 *
			 * @return Starting reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final reverse iterator
			 *
			 * @return Final reverse iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
			
		private:
			
			storage_type data; /**< List of pairs */
	};
	
	
	/** Tree
	 *
	 * Template for storing tree structures. This is a recursive
	 * structure storing the data and list of subtrees.
	 *
	 */
	template< class Tmemb> class Tree : public Class {
		public:
			
			typedef Tree< Tmemb> this_type;                                    /**< Template type alias */
			typedef List< this_type> subtrees_type;                            /**< Subtree list */
			typedef typename subtrees_type::size_type size_type;               /**< Size type */
			typedef typename subtrees_type::ref_type ref_type;                 /**< Subtree reference type */
			typedef Iterator< this_type, ref_type, this_type> iterator;        /**< Iterator */
			typedef RevIterator< this_type, ref_type, this_type> rev_iterator; /**< Reverse iterator */
			
			
			/** Constructor
			 *
			 * Create empty tree.
			 *
			 */
			Tree() {
				data = NULL;
			}
			
			
			/** Destructor
			 *
			 * Deallocate the tree.
			 *
			 */
			~Tree() {
				if (local)
					delete data; /* Delete data if local copy */
				/* Subtrees will be deallocated recursivelly  */
			}
			
			
			/** Check whether tree node is NULL
			 *
			 * Check whether key node is NULL, which
			 * also means that there are no subtrees.
			 *
			 * @return True if the node is NULL and has
			 *         no subtrees.
			 *
			 */
			bool IsNULL() const {
				return (data == NULL);
			}
			
			
			/** Get number of subtrees
			 *
			 * @return Number of subtrees.
			 *
			 */
			size_type CountSubtrees() const {
				return subtrees.Len();
			}
			
			
			/** Point node to data
			 *
			 * Point the node to the data. The node
			 * stores as pointer to the original data.
			 *
			 * @param prev Data to put to the node.
			 *
			 * @return Reference to this.
			 *
			 */
			const this_type & operator<<(const Tmemb * prev) {
				CB2_ASSERT(data == NULL, "Tree node can hold just one value");
				data = prev;
				local = false;
			}
			
			
			/** Push data to node
			 *
			 * Push the data to the node. The node
			 * stores local copy of the data, so a copy
			 * constructor is used.
			 *
			 * @param prev Data to push to the node.
			 *
			 */
			void Push(const Tmemb & prev) {
				CB2_ASSERT(data == NULL, "Tree node can hold just one value");
				data = new Tmemb(prev);
				local = true;
			}
			
			
			/** Create new subtree
			 *
			 * Create new subtree.
			 *
			 * @return New subtree of the node.
			 *
			 */
			this_type * AddChild() {
				Tree< Tmemb> child;
				subtrees.PushBack(child);
				return subtrees.Tail();
			}
			
			
			/** Get starting reference to subtrees
			 *
			 * @return Starting reference to subtrees.
			 *
			 */
			ref_type RefBegin() const {
				return subtrees.RefBegin();
			}
			
			
			/** Get final reference to subtrees
			 *
			 * @return Final reference to subtrees.
			 *
			 */
			ref_type RefEnd() const {
				return subtrees.RefEnd();
			}
			
			
			/** Get starting reverse reference to subtrees
			 *
			 * @return Starting reverse reference to subtrees.
			 *
			 */
			ref_type RevRefBegin() const {
				return subtrees.RevRefBegin();
			}
			
			
			/** Get final reverse reference to subtrees
			 *
			 * @return Final reverse reference to subtrees.
			 *
			 */
			ref_type RevRefEnd() const {
				return subtrees.RevRefEnd();
			}
			
			
			/** Get succeding reference
			 *
			 * @param Original reference.
			 *
			 * @return Succeding reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				return subtrees.RefNext(prev);
			}
			
			
			/** Get predecessing reference
			 *
			 * @param Original reference.
			 *
			 * @return Predecessing reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				return subtrees.RefPrev(prev);
			}
			
			
			/** Dereference
			 *
			 * @param Reference.
			 *
			 * @return Subtree.
			 *
			 */
			const this_type & Deref(const ref_type & ref) const {
				return subtree.Deref(ref);
			}
			
			
			/** Reverse dereference
			 *
			 * @param Reverse reference.
			 *
			 * @return Subtree.
			 *
			 */
			const this_type & RevDeref(const ref_type & ref) const {
				return subtree.RevDeref(ref);
			}
			
			
			/** Get starting subtree iterator
			 *
			 * @return Starting subtree iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final subtree iterator
			 *
			 * @return Final subtree iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting subtree reverse iterator
			 *
			 * @return Starting subtree reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final subtree reverse iterator
			 *
			 * @return Final subtree reverse iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
			
		private:
			Tmemb * data;           /**< Node data */
			bool local;             /**< Local copy flag */
			subtrees_type subtrees; /**< List of subtrees */
			
	};
	
	
	/** Split string to tokens
	 *
	 * This template splits string
	 * into tokens (delimited by another
	 * string), which are then accessible
	 * using iterators.
	 *
	 */
	template< class Tchar> class PStringTokenizer : public Class {
		public:
			
			typedef PStringTokenizer< Tchar> this_type;                           /**< Template type alias */
			typedef PString< Tchar> item_type;                                    /**< Item type */
			typedef List< item_type> storage_type;                                /**< Storage type */
			typedef typename storage_type::size_type size_type;                   /**< Size type */
			typedef typename storage_type::ref_type ref_type;                     /**< Reference type */
			typedef Iterator< this_type, ref_type, item_type> iterator;           /**< Iterator */
			typedef RevIterator< this_type, ref_type, item_type> rev_iterator;    /**< Reverse iterator */
			
			
			/** Constructor
			 *
			 * Initialize tokenizer with a string
			 * to split and delimiter. Prepare token
			 * strings.
			 *
			 * @param str   String to tokenize.
			 * @param delim Token delimiter.
			 *
			 */
			PStringTokenizer(const item_type & str, const item_type & delim) {
				tokens = new storage_type;
				
				size_type pos = 0;
				size_type len;
				size_type dlen = delim.Len();
				
				while ((pos < str.Len()) && (str.Pos(delim, len, pos))) {
					tokens->PushBack(str.Substr(pos, len - pos));
					pos = len + dlen;
				}
				
				tokens->PushBack(str.Substr(pos));
			}
			
			
			/** Destructor
			 *
			 * Delete tokens.
			 *
			 */
			~PStringTokenizer() {
				delete tokens;
			}
			
			
			/** Number of tokens
			 *
			 * @return Number of tokens.
			 *
			 */
			size_type Len() const {
				return tokens->Len();
			}
			
			
			/** Access token
			 *
			 * @param pos Index of the token to access.
			 *
			 * @return Token.
			 *
			 */
			const item_type & operator[](const size_type pos) const {
				return (*tokens)[pos];
			}
			
			
			/** Get starting reference
			 *
			 * Get the reference to the first
			 * token in the string.
			 *
			 * @return Starting reference.
			 *
			 */
			ref_type RefBegin() const {
				return tokens->RefBegin();
			}
			
			
			/** Get final reference
			 *
			 * Get the final reference. This is
			 * a reference pointing after
			 * the end of the string.
			 *
			 * @return Final reference.
			 *
			 */
			ref_type RefEnd() const {
				return tokens->RefEnd();
			}
			
			
			/** Get starting reverse reference
			 *
			 * Get the reference to the last
			 * token in the string.
			 *
			 * @return Starting reverse reference.
			 *
			 */
			ref_type RevRefBegin() const {
				return tokens->RevRefBegin();
			}
			
			
			/** Get final reverse reference
			 *
			 * Get the final reverse reference
			 * to the first item.
			 *
			 * @return Final reverse reference.
			 *
			 */
			ref_type RevRefEnd() const {
				return tokens->RevRefBegin();
			}
			
			
			/** Get succeding reference
			 *
			 * Get the reference to a token
			 * succeding the token referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Next reference.
			 *
			 */
			ref_type RefNext(const ref_type & prev) const {
				return tokens->RefNext(prev);
			}
			
			
			/** Get predecessing reference
			 *
			 * Get the reference to a token
			 * predecessing the token referenced
			 * by 'prev'.
			 *
			 * @param prev Original reference.
			 *
			 * @return Previous reference.
			 *
			 */
			ref_type RefPrev(const ref_type & prev) const {
				return tokens->RefPrev(prev);
			}
			
			
			/** Dereference
			 *
			 * Get the token (substring) referenced by 'ref'.
			 * For final or invalid references,
			 * Substr() method throws ExceptionRange.
			 *
			 * @param ref Token reference.
			 *
			 * @return Token value.
			 *
			 */
			const item_type & Deref(const ref_type & ref) const {
				return tokens->Deref(ref);
			}
			
			
			/** Reverse dereference
			 *
			 * Get the token (substring) referenced by 'ref'.
			 * For final or invalid references,
			 * Substr() method throws ExceptionRange.
			 *
			 * @param ref Token reference.
			 *
			 * @return Token value.
			 *
			 */
			const item_type & RevDeref(const ref_type & ref) const {
				return tokens->RevDeref(ref);
			}
			
			
			/** Get starting iterator
			 *
			 * @return Starting iterator.
			 *
			 */
			iterator Begin() const {
				return iterator(*this, RefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			iterator End() const {
				return iterator(*this, RefEnd());
			}
			
			
			/** Get starting reverse iterator
			 *
			 * @return Starting reverse iterator.
			 *
			 */
			rev_iterator RevBegin() const {
				return rev_iterator(*this, RevRefBegin());
			}
			
			
			/** Get final iterator
			 *
			 * @return Final iterator.
			 *
			 */
			rev_iterator RevEnd() const {
				return rev_iterator(*this, RevRefEnd());
			}
		
		private:
		
			storage_type * tokens; /**< String tokens */
		
	};
	

	typedef PString< char> String;                   /**< PString template specialization */
	typedef PStringTokenizer< char> StringTokenizer; /**< PStringTokenizer template specialization */
	typedef PMap< String, String> Map;               /**< PMap template specialization */
	typedef PMapList< String, String> MapList;       /**< PMapList template specialization */
	
	
	/** Generic ColorBox2 exception class
	 *
	 * Generic exception, other exceptions are derived
	 * from this class.
	 *
	 */
	class Exception : public Class {
		public:
			Exception();                                  /**< Constructor */
			Exception(const String what);                 /**< Constructor with description */
			Exception(const String what, const int line); /**< Constructor with description and line number */
			
			String What() const; /**< Exception description */
		private:
			String except; /**< Description */
	};
	
	
	/** Range checking exception
	 *
	 * This exception is thrown mostly in range
	 * check faults, i.e. when the range of a destination
	 * type is unable to carry the assigned value, etc.
	 *
	 */
	template< class Tsize> class ExceptionRange : public Exception {
		public:
			
			
			/** Constructor
			 *
			 * Exception without description.
			 *
			 */
			ExceptionRange()
				: Exception() {
				/* Empty */
			}
			
			
			/** Constructor
			 *
			 * Exception with description.
			 *
			 * @param what Description.
			 *
			 */
			ExceptionRange(const String what)
				: Exception(what) {
				/* Empty */
			}
			
			
			/** Constructor
			 *
			 * Exception with description and line number.
			 *
			 * @param what Description.
			 * @param line Line number.
			 *
			 */
			ExceptionRange(const String what, const int line)
				: Exception(what, line) {
			}
			
			
			/** Constructor
			 *
			 * Exception with complex description and line number.
			 *
			 * @param what  First part of the description.
			 * @param line  Line number.
			 * @param what2 Second part of the description.
			 *
			 */
			ExceptionRange(const String what, const Tsize value, const String what2, const int line)
				: Exception(what + String(value) + what2, line) {
			}
	};
	
	
	/** Conversion exception
	 *
	 * This exception is thrown when a run-time
	 * conversion from one type to another
	 * is not possible due to input or output
	 * limitations (i.e. when trying to convert
	 * a string not representing an integer to integer).
	 *
	 */
	class ExceptionConversion : public Exception {
		public:
			ExceptionConversion();                                  /**< Constructor */
			ExceptionConversion(const String what);                 /**< Constructor with description */
			ExceptionConversion(const String what, const int line); /**< Constructor with description and line number */
	};
	
	
	/** Memory exception exception
	 *
	 * This exception is thrown when a NULL
	 * pointer dereference or invalid pointer
	 * dereference should occur.
	 *
	 */
	class ExceptionMemory : public Exception {
		public:
			ExceptionMemory();                                  /**< Constructor */
			ExceptionMemory(const String what);                 /**< Constructor with description */
			ExceptionMemory(const String what, const int line); /**< Constructor with description and line number */
	};
	
	
	/** Assertion failed exception
	 *
	 * This exception is thrown when a critical
	 * condition is not true.
	 *
	 */
	class ExceptionAssert : public Exception {
		public:
			ExceptionAssert();                                  /**< Constructor */
			ExceptionAssert(const String what);                 /**< Constructor with description */
			ExceptionAssert(const String what, const int line); /**< Constructor with description and line number */
	};
	
	
	/** Item not found exception
	 *
	 * This exception is thrown when any method
	 * is supposed to find an item which cannot
	 * be found.
	 *
	 */
	class ExceptionNotFound : public Exception {
		public:
			ExceptionNotFound();                                  /**< Constructor */
			ExceptionNotFound(const String what);                 /**< Constructor with description */
			ExceptionNotFound(const String what, const int line); /**< Constructor with description and line number */
	};
	
};


#endif
