/***********************************************************************

  ColorBox2: Application.h
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.2			20th June 2003		basic structure
  0.3			20th August 2004	rewrite

***********************************************************************/


#ifndef __CB2__APPLICATION_H__
#define __CB2__APPLICATION_H__


#include <cb2/utils.h>
#include <cb2/Template.h>
#include <cb2/Widgets.h>
#include <iostream>


#define CB2_IMPLEMENT_APP_SER(AppType, SerType) \
	int main(int argc, char * argv[]) { \
		try { \
			SerType serializer(argc, argv, std::cin); \
			AppType app; \
		} catch (Exception & e) { \
			CB2::Application::AppError("Application exception", e.What()); \
		} catch (...) { \
			CB2::Application::AppError("Fatal application exception"); \
		} \
		\
		return 0; \
	}

#define CB2_IMPLEMENT_APP(AppType) CB2_IMPLEMENT_APP_SER(AppType, Serializer)


#define CB2_DECLARE_SERIALIZEDCLASS(ClassType) \
	virtual const char * GetSerialName();


#define CB2_IMPLEMENT_SERIALIZEDCLASS(ClassType) \
	const char * ClassType::GetSerialName() { \
		return #ClassType; \
	}


namespace CB2 {
	

	/** Handling persistent data storage
	 *
	 * A single instance of this class in the application
	 * handles the storage of serialized data from all persistent
	 * classes (i.e. classes inherited from SerializedClass).
	 *
	 */
	class Serializer : public Class {
		public:
			Serializer(int argc, char * argv[], std::istream & input);
			virtual ~Serializer();
		
			virtual void BeginInstance(const char * serialname, const char * instance);
			virtual const Serializer & operator<<(const String & data);
			virtual const Serializer & operator>>(String & data);
			virtual void EndInstance();
			
			virtual String Render();
			
			static Serializer & GetSerializer();
			
			MapList PostVars;
			MapList GetVars;
		
		protected:
			
			virtual void load(int argc, char * argv[], std::istream & input);
			
	};
	
	
	/** Basic class for classes with persistent data
	 *
	 * Classes inherited from this class can have
	 * persistent data to be stored during the application
	 * session (i.e. multiple runs of the application
	 * on the server machine).
	 *
	 * The inherited class must be declared with CB2_DECLARE_SERIALIZEDCLASS macro
	 * and contain the CB2_IMPLEMENT_SERIALIAZEDCLASS macro in the implementation
	 * part.
	 *
	 * Also the virtual methods Write() and Read() must be implemented
	 * to store and retrieve the persistent data from the serializer.
	 *
	 */
	class SerializedClass : public Class {
		public:
			SerializedClass(const char * instance);
			virtual ~SerializedClass();
		
			virtual bool Write(Serializer * storage) = 0;
			virtual bool Read(Serializer * storage) = 0;
			virtual const char * GetSerialName() = 0;
		
		private:
			
			const char * inst;
	};


	/** Main ColorBox2 application class
	 *
	 * A single instance of this class represents
	 * the main object in ColorBox2 application.
	 * By deriving a user Application class and
	 * overriding the constructor, any complex
	 * application can be created.
	 *
	 * Use macros CB2_IMPLEMENT_APP and CB2_IMPLEMENT_APP_SER
	 * to instantiate the application and give it control.
	 *
	 */
	class Application : public Class {
		public:
			Application();
			~Application();
			
			void SetTemplate(Template * temp, const String & anc);
			void SetMimeType(const String & mtype);
			
			Template * Register(Widget * child, String & anc);
			
			static void AppError(const String & error, const String & desc = "");
			
		private:
			Widget * member;
			Template * templ;
			String anchor;
			String mime;
	};
	
	
}


#endif
