/***********************************************************************

  ColorBox2: DS.h
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.4			10th September 2004		initial

***********************************************************************/


#ifndef __CB2__DS_H__
#define __CB2__DS_H__


#include <cb2/utils.h>
#include <mysql.h>


#define CB2_MYSQL(cond, error, warning) \
	if (!(cond)) \
		throw ExceptionMySQL(ExceptionMySQL::error, warning ": " #cond " in " __FILE__, __LINE__);


namespace CB2 {
	

	typedef MapList Row;       /**< Result row type */
	typedef List< Row > Query; /**< Result set type */
	
	
	/** Data query domain
	 *
	 * This class defines the domain for data
	 * queries. It is the way to define constrains
	 * on the domain of the query.
	 *
	 */
	class Domain : public Class {
		public:
			
			/** Constrain on string value
			 *
			 */
			typedef enum {
				str_row_eq, /**< Row value equals */
				str_row_neq /**< Row value not equals */
			} constrain_str_value;
			
			/** Constrain on integer value
			 *
			 */
			typedef enum {
				int_row_eq,  /**< Row value equals */
				int_row_neq, /**< Row value not equals */
				int_row_gt,  /**< Row value greater than */
				int_row_lt,  /**< Row value less than */
				int_row_ge,  /**< Row value greater or equal */
				int_row_le   /**< Row value less or equal */
			} constrain_int_value;
			
			Domain();                                                                        /**< Empty constrain */
			Domain(const Domain & prev);                                                     /**< Copy constructor */
			Domain(const constrain_str_value con, const String & key, const String & value); /**< Constrain on string value */
			Domain(const constrain_int_value con, const String & key, const int value);      /**< Constrain on string value */
			~Domain();                                                                       /**< Destructor */
			
			const Domain & operator&&(const Domain & right); /**< Merge constains using AND logical operator */
			const Domain & operator||(const Domain & right); /**< Merge constains using OR logical operator */
			const Domain & operator!();                      /**< Logical inversion of the constain */
			
			/** Constrain type
			 *
			 */
			typedef enum {
				str_value, /**< String value constrain */
				int_value  /**< Integer value constrain */
			} constrain_type;
			
			/** String value constrain specification
			 *
			 */
			typedef struct {
				constrain_str_value con; /**< Type of constrain */
				String key;              /**< Key to constrain */
				String value;            /**< Value to constrain */
			} constrain_str_value_atom;
			
			/** Integer value constrain specification
			 *
			 */
			typedef struct {
				constrain_int_value con; /**< Type of constrain */
				String key;              /**< Key to constrain */
				int value;               /**< Value to constrain */
			} constrain_int_value_atom;
			
			/** Atomic (single) constrain
			 *
			 */
			typedef struct {
				constrain_type type;                /**< Constrain type */
				constrain_str_value_atom str_value; /**< String value constrain data */
				constrain_int_value_atom int_value; /**< Integer value constrain data */
			} constrain_atom;
			
			typedef List< constrain_atom> constrain_literal; /**< Constrains literal (disjunction of atoms) */
			typedef List< constrain_literal> constrain_cnf;  /**< Constrains list (conjuction of literals) */

			constrain_cnf constrains; /**< Current constrains */
	};
	
	
	/** Generic data source interface
	 *
	 * Interface for working with data
	 * storages.
	 *
	 */
	class DataSource : public Class {
		public:
			DataSource();              /**< Constructor */
			virtual ~DataSource() = 0; /**< Destructor */
			
			virtual Query Select(const String & table, const Domain & domain, const Query::size_type limit, const Query::size_type displacement) = 0; /**< Select query */
	};
	
	
	/** MySQL exception
	 *
	 * Exception thrown on MySQL error
	 * (connection error, etc.).
	 *
	 */
	class ExceptionMySQL : public Exception {
		public:

			typedef enum {
				unknown_error,    /**< Unknown error */
				internal_error,   /**< Internal (client) error */
				connection_error, /**< Unable to connect to the server */
				query_error       /**< Query error */
			} error_type; /**< Kind of error */
		
			ExceptionMySQL();                                                          /**< Default constructor */
			ExceptionMySQL(const error_type error, const String what);                 /**< Costructor with error type and description */
			ExceptionMySQL(const error_type error, const String what, const int line); /**< Costructor with error type, description and line */
			
			error_type GetError() const; /**< Get error */
			
		private:
			
			error_type err; /**< Error occured */
	};
	
	
	/** MySQL data source
	 *
	 * Connect to MySQL database and work
	 * with the stored data.
	 *
	 */
	class DSMySQL : public DataSource {
		public:
			DSMySQL(const String & host, const String & user, const String & passwd, const String & db, const unsigned int port = 0, const String & socket = ""); /**< Connect */
			virtual ~DSMySQL();                                                                                                                                   /**< Disconnect */
			
			Query Select(const String & table, const Domain & domain, const Query::size_type limit = 0, const Query::size_type displacement = 0); /**< Select query */
			
		private:
			
			MYSQL * handle; /**< Connection handle */
			
	};
	
	
	/** Flat file data source
	 *
	 * Open a flat binary file and retrieve
	 * data from it.
	 *
	 */
	class DSFlatFile : public DataSource {
		public:
			DSFlatFile(const String & filename);
			virtual ~DSFlatFile();
			
			Query Select(const String & table, const Domain & domain, const Query::size_type limit = 0, const Query::size_type displacement = 0); /**< Select query */
	};
	
	
	/** CSV file data source
	 *
	 * Open a CSV text file and retrieve
	 * data from it.
	 *
	 */
	class DSCSVFile : public DataSource {
		public:
			DSCSVFile(const String & filename);
			virtual ~DSCSVFile();
			
			Query Select(const String & table, const Domain & domain, const Query::size_type limit = 0, const Query::size_type displacement = 0); /**< Select query */
	};
	
	
	/** XML file data source
	 *
	 * Open a XML file and retrieve
	 * data from it.
	 *
	 */
	class DSXMLFile : public DataSource {
		public:
			DSXMLFile(const String & filename);
			virtual ~DSXMLFile();
			
			Query Select(const String & table, const Domain & domain, const Query::size_type limit = 0, const Query::size_type displacement = 0); /**< Select query */
	};
	
}


#endif
