/***********************************************************************

  ColorBox2: Widgets.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			20th August 2004	initial

***********************************************************************/


#include <cb2/Widgets.h>
#include <cb2/Application.h>


using namespace CB2;


/** Constructor
 *
 * Create size value.
 *
 * @param value Scalar value.
 * @param type  Unit.
 *
 */
Size::Size(const int value, const size_type type) {
	svalue = value;
	stype = type;
}


/** Get size value
 *
 * @return Size value (scalar and unit).
 *
 */
String Size::GetSize() const {
	String ret;
	
	ret = svalue;
	
	/* Fill unit */
	if (svalue != 0) {
		switch (stype) {
			case px:
				ret += " px";
				break;
			case em:
				ret += " em";
				break;
			default:
				CB2_ASSERT(false, "Unknown size type");
		}
	}
	
	return ret;
}


/** Constructor
 *
 * Create a new widget as a nested object
 * of a container, register as a child.
 *
 * @param parent Container containing this widget.
 *
 */
Widget::Widget(Container & parent) {
	ptempl = parent.Add(this, panchor);
	filled = false;
}


/** Constructor
 *
 * Create a new widget as a nested object
 * of an application, register as a child.
 *
 * @param parent Application containing this widget.
 *
 */
Widget::Widget(Application & parent) {
	ptempl = parent.Register(this, panchor);
	filled = false;
}


/** Render widget
 *
 * Render widget contents to the parent
 * template.
 *
 * @param data Widget contents.
 *
 */
void Widget::Data(const String & data) {
	filled = true;
	if (ptempl)
		ptempl->Subst(panchor, data);
}


/** Destructor
 *
 * If the widget was not rendered
 * to the parent template, ExceptionAssert
 * is thrown.
 *
 */
Widget::~Widget() {
	CB2_ASSERT(filled, "Widget didn't supply its data to parent");
}


/** Copy constructor
 *
 * Copying of widgets is not allowed, Exception
 * is thrown.
 *
 * @param prev Widget to copy from.
 *
 */
Widget::Widget(const Widget & prev) {
	CB2_ASSERT(false, "Widget cannot use copy constructor");
}


/** Assignment operator
 *
 * Copying of widgets is not allowed, Exception
 * is thrown.
 *
 * @param prev Widget to copy from.
 *
 */
void Widget::operator=(const Widget & prev) {
	CB2_ASSERT(false, "Widget cannot use assignment operator");
}


/** Constructor
 *
 * Create a container.
 *
 * @parent Container containing this container.
 *
 */
Container::Container(Container & parent)
	: Widget(parent) {
	/* Empty */
}


/** Constructor
 *
 * Create a container.
 *
 * @parent Application containing this container.
 *
 */
Container::Container(Application & parent)
	: Widget(parent) {
	/* Empty */
}


/** Constructor
 *
 * Create a spacer defined by CSS class
 * and/or id.
 *
 * @param parent    Container containing this widget.
 * @param css_class CSS class of this widget.
 * @param css_id    CSS id of this widget.
 *
 */
Spacer::Spacer(Container & parent, const String & css_class, const String & css_id)
	: Widget(parent) {
	init(css_class, css_id);
}



/** Constructor
 *
 * Create a spacer defined by CSS class
 * and/or id.
 *
 * @param parent    Application containing this widget.
 * @param css_class CSS class of this widget.
 * @param css_id    CSS id of this widget.
 *
 */
Spacer::Spacer(Application & parent, const String & css_class, const String & css_id)
	: Widget(parent) {
	init(css_class, css_id);
}
			

/** Constructor
 *
 * Create a spacer defined by its height.
 *
 * @param parent Container containing this widget.
 * @param height Height of the spacer.
 *
 */ 
Spacer::Spacer(Container & parent, const Size & height)
	: Widget(parent) {
	init(height);
}


/** Constructor
 *
 * Create a spacer defined by its height.
 *
 * @param parent Application containing this widget.
 * @param height Height of the spacer.
 *
 */ 
Spacer::Spacer(Application & parent, const Size & height)
	: Widget(parent) {
	init(height);
}
			

/** Destructor
 *
 * Render the spacer.
 *
 */
Spacer::~Spacer() {
	Data(data);
}


/** Initialize the spacer
 *
 * Initialize a spacer defined by CSS class
 * and/or id.
 *
 * @param css_class CSS class of this widget.
 * @param css_id    CSS id of this widget.
 *
 */
void Spacer::init(const String & css_class, const String & css_id) {
	if ((css_class != "") || (css_id != "")) {
		data = "<div";
		
		if (css_id != "")
			data += String(" id=\"") + css_id.HTML() + String("\"");
		
		if (css_id != "")
			data += String(" class=\"") + css_class.HTML() + String("\"");
		
		data += "></div>";
	}
}


/** Initialize the spacer
 *
 * Initialize a spacer defined by its height.
 *
 * @param parent Application containing this widget.
 *
 */
void Spacer::init(const Size & height) {
	data = String("<div style=\"height: ") + height.GetSize().HTML() + String(";\"></div>");
}
